SublimeHaskell 1.x.y Consolidated Release Messages
==================================================

## 1.2.5

- Relax how subprocesses inherit _SublimeHaskell_'s augmented environment. _SublimeHaskell_ now makes a copy of the current environment and only updates the _PATH_. This allows you to change `os.environ` in the ST console and have those changes propagate to _SublimeHaskell_ subprocesses. (__NOTE:__ Changing `os.environ` won't impact the `hsdev` backend, but it will affect how `stack` works, if, for example, you add or set `os.environ['http_proxy']`.)

## 1.2.4

- Fixed an issue with the `add_to_PATH` setting.

- Fixed an issue with error navigation (click on error in the compile output, go to source.)

- Add _SublimeHaskell_ to the SublimeText build system. `&lt;Ctrl&gt;-&lt;Primary&gt;-B` or the `Tools&gt;Build with...` menu now lists the available _SublimeHaskell_ build tasks. They're still available in the `&lt;Ctrl&gt;-&lt;Shift&gt;-P` command palette. For now...

__NOTE:__ You may encouter issues with newer version of `hsdev`, which use `hlint-2.0` or newer. `hlint-2.0`-`hlint-2.0.5` do not work, and `hsdev.cabal` may actually use `hlint-2.0` during the configure phase. If _SublimeHaskell_ inexplicably pauses during _Check and Lint_ or _Lint_, you will have to manually edit `hsdev.cabal` to ensure that the end of the `hlint` version range is `&lt;2` and not `&lt;2.0`. Recompile, reinstall `hsdev` and the issue should be fixed.

## 1.2.3

- Break fix: Use `re.search` instead of `re.match` to detect local `hsdev` server's startup. New issue based on recent `hsdev` changes.

## 1.2.2

- Make the parsed error/warning output window reappear after a build. (Reported by _cartazio_.)

- _Ctrl-B_ invokes `SublimeHaskell: Build`. No more spurious error messages about `runhaskell`. (Updated Mac OS X keymap is in the works so that _Cmd-B_ does the right thing too. Please have some patience.)

- Bump the `hsdev` version number _and_ patch the command line to reflect changes to the `simple-log` package's updated (simplified) command line arguments. __Please read the SublimeHaskell package's default settings if you want or need to adjust the `hsdev` logging level.__

- Add support for the new `cabal` build system (see the _haskell_build_tool_ setting.) The builder's name is `cabal-new-build`. Eventually, this will become `cabal` when the `cabal` maintainers decide their new build system is stable.

## 1.2.1

- Fix `client.py` tracebacks and `hsdev` connection sequencing. Connections are now synchronous, not asynchronous.

## 1.2.0

_SublimeHaskell_ 1.2.0 is another stability release, with a lot of fixes resulting from a `pylint` code audit. While a majority of these "fixes" were Python style-related (line lengths, spaces between keywords and arguments), there were errors (usually bad attribute references to the wrong imported module) and a need to simplify complex logic in a few places (breaking up complex `if`-`elif`-`else` blocks.) The overall McCabe assessment went from 5.75/10 to 9.79/10, a significant improvement.

### SublimeText 2 Support Removed

SublimeText 2 compatibility was tested after the `pylint` code audit. The current code base uses API calls to the SublimeText3 API that aren't present or featured in SublimeText2, and no workarounds exist. Couple that with the complete lack of issues related to SublimeText 2 and it wasn't a hard decision to remove SublimeText 2 compatibility.

SublimeText 2: We loved you. We bid you a fond farewell as you head to your retirement.

### New Feature: Remote `hsdev`

The `hsdev` backend has three new preferences that control _SublimeHaskell_'s interaction with the `hsdev` backend:

| Preference            | Description                                                           |
| --------------------- |-----------------------------------------------------------------------|
| `hsdev_host`          | The host name where the `hsdev` backend is located.                   |
|                       | Default: "localhost"                                                  |
| `hsdev_port`          | The port number to which the `hsdev` backend listens for connections. |
|                       | Default: 4567                                                         |
| `hsdev_local_process` | Controls whether _SublimeHaskell_ will start up a local `hsdev`       |
|                       | backend process or simply connect to the backend via `hsdev_host`     |
|                       | and `hsdev_port`.                                                     |
|                       | Default: true (`hsdev` backend process started by _SublimeHaskell_)   |

Add and customize the folowing SublimeHaskell_ "Settings - User" preferences if you have a remote `hsdev` backend that you want to use instead of having _SublimeHaskell_ start up its own `hsdev`:

```
{
    // Remote hsdev is on the "my_hsdev_host.domain.net" machine
    "hsdev_host": "my_hsdev_host.domain.net",
    // Remote hsdev listens to port 4096 for connections (default: 4567)
    "hsdev_port": 4096,
    // SublimeHaskell will not start or manage its own hsdev backend process
    "hsdev_local_process": false,

    // Other preferences follow...
}
```

### Next/Previous Error

These commands now work as intended. "Goto next error" didn't; it would get stuck at the first error.

The "<alt>-D <alt>-E" key mapping for "next error" and "<shift>-<alt>-D <shift>-<alt>-E" mapping for "previous error" can be a bit awkward. This key mapping was borrowed from the (now defunct) SublimeClang plugin. If you prefer "<ctrl>-K n" and "<ctrl>-K p" that SublimeLinter uses, add these two lines to your "Key Bindings" preferences:

```
  { "keys": ["ctrl+k", "n"], "context": [ { "key": "haskell_source" } ], "command": "sublime_haskell_next_error" },
  { "keys": ["ctrl+k", "p"], "context": [ { "key": "haskell_source" } ], "command": "sublime_haskell_previous_error" },

```

### In Case You Missed It...

#### Environment variables in `add_to_PATH`

You may have missed this in an earlier release: You can use environment variables in the `add_to_PATH` setting. Both Unix-style (`$HOME`) and Windows-style (`%APPDATA%`) are supported. Tilde-expansion (`~/.local/bin`) also works.

#### `add_standard_dirs` Setting

_SublimeHaskell_ will add several "standard" directories to its `PATH` when searching for tools. This eliminates the need to modify the `add_to_PATH` setting if you installed `hsdev` or `ghc-mod` in one of the "standard" Haskell places. `add_standard_dirs` defaults to `true`.

Excerpt from the default setings:

```
    // Add the "standard" directories where cabal and stack install executables,
    // if they exist, in the following order (top to bottom):
    //
    // stack's install dir
    // user bindir from $HOME/.cabal/config (*nix) or  %APPDATA%/cabal/config (WinXX)
    // global bindir from $HOME/.cabal/config (*nix) or %APPDATA%/cabal/config (WinXX)
    //
    // The defaults are:
    //
    // stack:  {$HOME/.local|%APPDATA%/local}/bin
    // user:   {$HOME/.cabal|%APPDATA%/cabal}/bin
    // global: {/usr/local|%PROGRAMFILES%/Haskell}/bin
    "add_standard_dirs": true,
```

Basically, this should make it easier to use _SublimeHaskell_ out-of-the-box.

### Upcoming Development Milestones

Upcoming development milestones, listed in order of priority:

#### Backend Refactoring (Expected: 1.4.0)

Work is in progress to refactor _SublimeHaskell_'s interaction with the three backends: `hsdev`, `ghc-mod` and `hdevtools`.  Support for `hdevtools` may be dropped completely -- if `hdevtools` is important to you, please file an issue. The rationale for this refactoring is to make the interface cleaner between the plugin and the backend so that it's clear what features the backend supports and provide a reasonable default action for unsupported features. This will also enable enterprising Haskell backend developers to easily add new backends to _SublimeHaskell_ (e.g., `intero` support has come up a few times as an enhancement request.)

__Note__: The refactored backend will introduce __BREAKING CHANGES__ to the default and user settings.

#### Lazy Backend Startup (Expected: 1.4.0)

Related to backend refactoring is lazy backend startup: _SublimeHaskell_ should start up and connect to a backend only when it's needed. Currently, _SublimeHaskell_ is too eager and starts up the `hsdev` backend when the plugin is loaded. This enhancement will make _SublimeHaskell_ lazier or, at least, less eager.

#### Switching between multiple `stack.yaml`-s (Expected: 1.3.0)

For `stack` tool users, it will be possible to specify a list of `stack.yaml` files in your project settings and switch between them. When you switch, _SublimeHaskell_ will execute a rebuild (`clean` followed by `build`).

#### The SublimeHaskell Editing Guide (Ongoing)

Documentation is always a good thing. The guide is a work-in-progress.

#### SublimeText 3 Syntax Files (Expected: 1.5.0)

There are numerous issues related to syntax highlighting. Syntax highlighting in SublimeText 2 was driven by collections of regular expressions. SublimeText 3 syntax highlighting still uses regexps and offers the ability to be more stateful. However, transitioning from _SublimeHaskell_'s improved ST2 syntax highlighting to ST3 syntax highlighting requires substantial work.

## 1.1.0

This is a stability release. The main fix is smoother and more reliable interaction between _SublimeHaskell_ and the `hsdev` process. No lockups due to unread data in the pipes between SublimeHaskell and `hsdev`. This fix also improved the reliability of command that communicate with a subprocess, such as indenters.

### New Indenter: `hindent`

_SublimeHaskell_ now supports `hindent` via the _"SublimeHaskell: HIndent"_ command. Select a region and use your favorite indenter -- `stylish-haskell` or `hindent`. If you don't select a region, the indenter stylizes the entire buffer's contents.

### More Reactive Output

Output from tools and builds is now more "reactive", that is, you can see what's going on while the tool executes. Previously, _SublimeHaskell_ appeared to stall and presented the tool's or build's output when the command completed. Output appears in the build window as it is received.

### Issues closed

- 319:  "GHCI current file" does not work in current release.
- 316:  stack exec fails
- 314:  build on save no longer works with current release
- 308:  build seems to hang
- 307:  SublimeHaskell crashing on OS X
- 303:  Highlighting Issue with Indented Guards
- 302:  Add well known paths enhancement
- 298:  hsdev IO exception in stack-based project
- 294:  Disabling SH's built-in Syntaxes
- 291:  Syntax highlighting with single-line {-multi--line-} comment bug syntax
- 289:  can_complete_qualified_symbol not defined on every autocomplete action
- 281:  Command for stack exec needed to show output. enhancement

### Behind The Scenes: Refactoring

`sublime_haskell_common.py` was partially refactored into smaller logical and atomic modules with related functionality. The intent is to make it easier to scan and read the code, as well as simplify Python import dependencies. More refactoring in following releases...

## 1.0.3

With SublimeText 3 dev > 3070 you can now hover mouse over symbol to get details popup, and over gutter with error/warning/hint icon to get detailed message.

## 1.0.2

Icons for errors, warnings and hints added. Also try `Hasky (Dark)` theme for better coloring of haskell entities and errors/warnings/hints.

## 1.0.1

Command `Toggle symbol info panel` added, which shows/hides continuous symbol info panel for symbol under selection. It may be useful for reading code with no need to call `Show symbol info` each time.

## 1.0.0

`hsdev`  branch is now merged into `master`. There are many changes, so it's highly recommended to read updated [README](https://github.com/SublimeHaskell/SublimeHaskell/)

You have to install `hsdev` package (`cabal install hsdev`; will be available on stack soon too), add it to `PATH` (or by using `add_to_PATH` setting) and then you'll be able to use all `SublimeHaskell: ...` commands.
