*SublimeHaskell* 2.1.10
=======================

SublimeHaskell 2.1.x is a release tree for minor enhancements and fixes to newly identified (and resolved) issues.

__RESTART SublimeText AFTER EACH UPDATE!!__ This ensures that stale versions of internal *SublimeHaskell* Python modules don't cause problems when *PackageControl* reloads the plugin. *SublimeHaskell* does not (and cannot) forcibly reload its own internal modules.

## In this release...

**Break fixes**:

- Fix a traceback in `HsDevBackend.files_and_contents`; `None` is usually passed in as a default, not an empty dictionary.

- Revert some `sublime_plugin.ViewEventListener` refactoring, since the API was only recently "enhanced" (Build 3155 -- most users are on Build 3146.) 

## 2.1.9, 2.1.9.1

**Fly mode**: Checking or linting after a pause in typing, aka "fly mode", now works reliably again. To enable, add the following to your user settings:

    "lint_check_fly": true

You can control the idle time that _SublimeHaskell_ waits before initiating checking or linting via the `lint_check_fly_idle ` setting. The default is 5 seconds. You should not set this to a value lower than 3 seconds or checks/lints can occur faster than you can type (although there is nothing to stop you from doing so in the code. Yet.)

**Issue 374 (enhancement)**: Limiting the error, warning and hints displayed in the _Sublime Haskell Output Window_ after a check, lint or build is now supported via the `show_only` setting:

    "show_only": {
        "errors": true,
        "warnings": true,
        "hints": true
    },

The `show_only` setting can be customized as _SublimeHaskell_ project setting, i.e., from within a `.sublime-project` file:

    {
      "folders":
      [
        {
          "path": "."
        }
      ],
      "SublimeHaskell":
        {
          "show_only": {
              "errors": true,
              "warnings": true,
              "hints": true
          },
        }
    }

## 2.1.8

**Issue 365**: Rethink how plugin preferences are stored in the `SettingsContainer` object; use Python properties instead of being clever with instance attributes. Property setters allow validity testing when new preference values are read.

## 2.1.7

**Issue 377 (possible 373, 359)**: @coodoo noticed a `UnicodeDecodeError` traceback after saves, which turns out to be a short read from the backend's socket -- more data is available, which SublimeHaskell should continue to read in order to ensure that decode('utf-8') succeeds. The fix here is to continue reading from the socket if decode('utf-8') raises the UnicodeDecodeError exception.  If no exception is raised, then split and process what looks like a complete request.

This probably also fixes issues 373 and 359, which manifested similar behavior, but without the traceback.

## 2.1.6

**Issues 375, 376 (duplicate)**: Make the "local", "exec-with" and "install-dir" `hsdev` backend options work together properly. If, for some reason, a backend has "local" set but either "exec-with" or "install-dir" isn't set, SublimeHaskell should not generate a backtrace. It should just mark the particular backend as unusable and continue onward.

## 2.1.5

**Fix REPL support**: REPL support was supported solely via `cabal repl` and the code supporting Haskell REPLs itself needed re-evaluation. All execution wrappers are now supported: `cabal repl`, `cabal new-repl` and `stack repl`.

*NOTE: The `haskell_build_tool` setting determines which execution wrapper is used. `haskell_build_tool` can be a project-specific setting, not just a default or user preference. See below in Release 2.1.3.*

**Fix status line issues**: There was a long standing bug with SublimeHaskell's use and manipulation of the status line. The main manifestation was a thread that repeatedly updated the status line every 0.2 seconds -- it wasn't always perceivable because there wasn't any noticeable flicker. This fix should cut down in CPU and thread usage that this issue may have caused.

**'SublimeHaskell: Reinspect All' now asynchronous**: This command now does its work asynchronously. Previously, it did its work in the SublimeText UI thread, leading to perceived hangs.

## 2.1.4

**Issue 371**: Revert a spurious change to a `hsdev` backend method (that was supposed to be reverted.)

## 2.1.3

**SublimeHaskell: Run**: Make running project executables play nicely with execution wrappers (e.g., `cabal run`, `stack exec`, `cabal new-run`). Eliminated the 'SublimeHaskell: Stack Exec' item because all of the functionality is included in 'SublimeHaskell: Run'. This change eliminates all of the previous gymnastics required to locate the executable in the first place.

`SublimeHaskell: Run` will also prompt for command line arguments, which are parsed into a Python list by the `shlex` package. Arguments will persist across command invocations and Sublime Text sessions.

**Issue 365**: More linting on the `prettify_exectable` setting. Someday, we'll know where the "{0}" comes from.

**Issue 361**: Add `cabal new-test` support in the `SublimeHaskell: Test` command. Restored automatic testing after a successful build; automatic benchmarking is also possible after a successful build. See **auto_build_mode** note below.

**Issue 344**: Most, if not all, 'cabal new-*' commands should be available, including `new-freeze` and `new-bench`. Corresponding commands for regular `cabal` and `stack` (as applicable) implemented. The major hack is that `cabal-new-build` does not have a clean command, so this is implemented as a function inside the SublimeHaskell builder.

**auto_build_mode change**: The `auto_run_tests` setting is now deprecated. Its functionality is now replaced by two new build modes: *normal-then-test* (run tests after a successful build) and *normal-then-bench* (run benchmarks after a successful build.) SublimeHaskell will pop up an information dialog if `auto_run_tests` is in your preferences.

**Project Settings**: All SublimeHaskell-related project settings are now contained within the `SublimeHaskell` dictionary. This allows you to encapsulate certain settings, like `haskell_build_tool`, on a per-project basis instead of via user preferences. There is more information at the bottom of the `SublimeHaskell.sublime-settings` (default plugin settings) file.

    {
      "folders":
      [
        {
          "path": "."
        }
      ],
      "SublimeHaskell": {
        "haskell_build_tool": "cabal-new-build"
      }
    }

**build.py**: Code cleanups related to building Haskell applications, mostly related to better class structure, fewer global variables. (Related to issues 361 and 344.)

**`pylint` cleanups**: `pylint` has updated, lots of new warnings. Fix code to bring them down to a dull roar.

## 2.1.2

**Issue 366**: Word and default completions did not produce results, which became apparent while using the "none" backend. Removed the `inhibit_completions` setting because its purpose was confusing. Added two new settings, `add_word_completions` and `add_default_completions`. `add_word_completions` adds words in the source file that are more than three characters; this is a SublimeText feature that SublimeHaskell normally inhibits. `add_default_completions` adds the contents of the `Haskell.sublime-completions` file, which SublimeHaskell normally inhibits as well. Both settings default to `false`. (H/T to @Pastafarianist.)

## 2.1.1

**ghc-mod**: Completions now work again, modeled off the `ghc-mod/elisp/ghc-comp.el` code. It's not perfect, but it suffices to improve developer productivity.

**Issue 365 (partial)**: Fix dignostic message when SublimeHaskell doesn't accept the `prettify_executable` setting's value.

**Issue 364**: `SublimeHaskell: Test` actually invoked the `install` task, not the `test` task. Also added `SublimeHaskell - Test` to the "Build With..." menu. Issue fixed.

## 2.1.0

**Issue 352**: Add command line options for `stylish-haskell` and `hindent`. NOTE: These are lists, not strings.

## Upcoming Development Milestones

Upcoming development milestones, listed in order of priority:

### SublimeText 3 Syntax Files (Expected: 2.2.0)

There are numerous issues related to syntax highlighting. Syntax highlighting in SublimeText 2 was driven by collections of regular expressions. SublimeText 3 syntax highlighting still uses regexps and offers the ability to be more stateful. However, transitioning from _SublimeHaskell_'s improved ST2 syntax highlighting to ST3 syntax highlighting requires substantial work.

### The SublimeHaskell Editing Guide (Ongoing)

Documentation is always a good thing. The guide is a work-in-progress.
